import sys
import time
import utils
import longvideo_config
import logging

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s',
                    datefmt='%a, %d %b %Y %H:%M:%S')


def remove_instances(create_client, alb_client, instance_ids):
    """
    停止并释放机器
    :param create_client:
    :param alb_client:
    :param instance_ids: instanceId type-list
    :return: None
    """
    # 1. 摘除 ALB
    for server_group_id in longvideo_config.server_group_id_list:
        utils.remove_servers_from_server_group(alb_client=alb_client, server_group_id=server_group_id,
                                               instance_id=instance_ids)
    logging.info(f"remove from ALB finished, instances: {instance_ids}")

    time.sleep(10)

    # 2. 停止机器
    utils.stop_instances(client=create_client, instance_ids=instance_ids)
    logging.info(f"instances stop finished, instances: {instance_ids}")

    # 3. 判断机器运行状态是否为 Stopped
    while True:
        response = utils.get_instances_status(client=create_client, instance_ids=instance_ids)
        if response.get('Code') is None:
            instances_list = response.get('InstanceStatuses').get('InstanceStatus')
            stopped_instances = [instance.get('InstanceId') for instance in instances_list if
                                 instance.get('Status') == 'Stopped']
            if len(stopped_instances) == len(instance_ids):
                logging.info(f"instances stopped status set success, instances: {stopped_instances}")
                break
            else:
                logging.info(f"stopped instances count = {len(stopped_instances)}, instances: {stopped_instances}")
                time.sleep(5)
        else:
            logging.error(response)
            sys.exit()

    # 4. 释放机器
    response = utils.release_instances(client=create_client, instance_ids=stopped_instances)
    if response.get('Code') is None:
        logging.info(f"release instances finished, instances: {stopped_instances}")
    else:
        logging.error(f"release instances fail!!!")
        sys.exit()


def main():
    try:
        alb_client = utils.connect_alb_client(
            access_key_id=longvideo_config.alb_client_params['access_key_id'],
            access_key_secret=longvideo_config.alb_client_params['access_key_secret']
        )
        create_client = utils.connect_client(
            access_key_id=longvideo_config.create_client_params['access_key_id'],
            access_key_secret=longvideo_config.create_client_params['access_key_secret'],
            region_id=longvideo_config.create_client_params['region_id']
        )

        # 获取指定释放的机器数量
        reduce_count = int(sys.argv[1])
        logging.info(f"reduce instances count: {reduce_count}")

        # 获取 ALB 下所有机器
        online_instance_ids = utils.get_instance_ids(client=alb_client, server_group_id=longvideo_config.server_group_id_list[0])
        online_instance_count = len(online_instance_ids)
        logging.info(f"online instance count: {online_instance_count}.")
        logging.info(f"online instance ids: {online_instance_ids}")

        # 获取前 count 台机器进行释放
        reduce_instance_ids = online_instance_ids[:reduce_count]
        logging.info(f"reduce instances: {reduce_instance_ids}")

        # 停止并释放机器
        remove_instances(create_client=create_client, alb_client=alb_client, instance_ids=reduce_instance_ids)
        logging.info(f"stop & release instances end!")
    except Exception as e:
        logging.error(e)
        sys.exit()



if __name__ == '__main__':
    main()
